package net.meutel.swarm.etcd;

import org.wildfly.swarm.config.runtime.AttributeDocumentation;
import org.wildfly.swarm.spi.api.Defaultable;
import org.wildfly.swarm.spi.api.Fraction;
import org.wildfly.swarm.spi.api.annotations.Configurable;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * Fraction Etcd Topology.
 */
@Configurable("net.meutel.etcd")
public class EtcdTopologyFraction implements Fraction<EtcdTopologyFraction> {

    private static final URL DEFAULT_URL;

    @AttributeDocumentation("Etcd URL")
    private Defaultable<URL> url = Defaultable.url(DEFAULT_URL);

    static {
        URL tmp = null;
        try {
            tmp = new URL("http://localhost:2379");
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }
        DEFAULT_URL = tmp;
    }

    public EtcdTopologyFraction() {
        this(DEFAULT_URL);
    }

    public EtcdTopologyFraction(URL url) {
        this.url.set(url);
    }

    public EtcdTopologyFraction(String url) throws MalformedURLException {
        this.url.set(new URL(url));
    }

    public URL url() {
        return this.url.get();
    }
}
