package net.meutel.swarm.etcd.runtime;

import io.vertx.core.Vertx;
import io.vertx.core.http.HttpClient;
import io.vertx.core.http.HttpClientOptions;
import io.vertx.core.http.HttpClientResponse;
import org.jboss.msc.service.*;
import java.net.URL;

/**
 * Etcd Client.
 */
public class EtcdClient implements Service<EtcdClient> {

    public static final ServiceName SERVICE_NAME = ServiceName.of("net", "meutel", "swarm","topology", "etcd", "client");

    HttpClient httpClient;

    URL url;

    public EtcdClient(URL url) {
        this.url = url;
    }

    public void createInDir(String dir, String value) {
        String body = "value=" + value;
//        String body = "nimp";
        System.out.println("  - createInDir: " + body);
        this.httpClient.post("/v2/keys/" + dir, this::parseResponse)
                .exceptionHandler(throwable -> {
                    System.err.println("EtcdClient error");
                    throwable.printStackTrace();
                })
//                .setChunked(true)
                .putHeader("Content-Length", String.valueOf(body.length()))
                .putHeader("Content-Type", "application/x-www-form-urlencoded")
                .write(body)
                .end();
    }

    private void parseResponse(HttpClientResponse resp) {
        int st = resp.statusCode();
        if (st < 200 || (st >= 300 && st < 400)) {
            System.err.println("Unexpected status: " + st);
        } else if (st >= 400 && st < 500) {
            System.err.println("Request error: " + st+ " (" + resp.statusMessage() + ")");
        } else if (st >= 500) {
            System.err.println("Etcd error: " + st+ " (" + resp.statusMessage() + ")");
        }
        resp.handler(buffer -> System.out.print("  = " + buffer));
    }

    @Override
    public void start(StartContext startContext) throws StartException {
        System.out.println("Starting EtcdClient for " + this.url);
        this.httpClient = Vertx.vertx()
                .createHttpClient(new HttpClientOptions()
                        .setLogActivity(true)
                        .setDefaultHost(this.url.getHost())
                        .setDefaultPort(this.url.getPort())
                        .setConnectTimeout(1000)
                        .setMaxPoolSize(10));
    }

    @Override
    public void stop(StopContext stopContext) {

    }

    @Override
    public EtcdClient getValue() throws IllegalStateException, IllegalArgumentException {
        return this;
    }
}
