package net.meutel.swarm.etcd.runtime;

import org.jboss.as.network.SocketBinding;
import org.jboss.msc.inject.Injector;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;
import org.jboss.msc.value.InjectedValue;
import org.wildfly.swarm.topology.TopologyConnector;
import org.wildfly.swarm.topology.runtime.Registration;
import org.wildfly.swarm.topology.runtime.TopologyManager;

import java.net.URL;

/**
 * Topology connector for etcd.
 */
public class EtcdTopologyConnector implements Service<EtcdTopologyConnector>, TopologyConnector {

    private static String TOPOLOGY_SOURCE_KEY = "etcd";

    private EtcdClient client;

    private InjectedValue<TopologyManager> topologyManagerInjector = new InjectedValue<>();

    private InjectedValue<EtcdClient> clientInjector = new InjectedValue<>();

    public Injector<EtcdClient> getClientInjector() {
        return this.clientInjector;
    }

    public EtcdTopologyConnector getValue() throws IllegalStateException, IllegalArgumentException {
        return this;
    }

    public Injector<TopologyManager> getTopologyManagerInjector() {
        return this.topologyManagerInjector;
    }

    public void start(StartContext startContext) throws StartException {
        System.out.println("Starting EtcdTopology");
        this.client = this.clientInjector.getValue();
    }

    public void stop(StopContext stopContext) {
        this.client = null;
    }

    public synchronized void advertise(String name, SocketBinding binding, String... tags) throws Exception {
        Registration registration = new Registration(TOPOLOGY_SOURCE_KEY, name, binding.getAddress().getHostAddress(), binding.getAbsolutePort(), tags);
        System.out.println("registration: " + registration);

        this.topologyManagerInjector.getValue().register(registration);

        URL url = new URL("http", registration.getAddress(), registration.getPort(),"");
        System.out.println("URL for service " + name + ": " + url);
        client.createInDir(name, url.toExternalForm());
    }

    public synchronized void unadvertise(String name, SocketBinding binding) throws Exception {

    }
}
